@@ -43,10 +43,9 @@ with `try/except` statement. This is required for consistent handling of
 database errors.
 """
 
-import six
+from oslo_utils.excutils import CausedByException
 
 from oslo_db._i18n import _
-from oslo_utils.excutils import CausedByException
 
 
 class DBError(CausedByException):
@@ -59,7 +58,7 @@ class DBError(CausedByException):
 
     def __init__(self, inner_exception=None, cause=None):
         self.inner_exception = inner_exception
-        super(DBError, self).__init__(six.text_type(inner_exception), cause)
+        super(DBError, self).__init__(str(inner_exception), cause)
 
 
 class DBDuplicateEntry(DBError):
@@ -192,18 +191,7 @@ class DBInvalidUnicodeParameter(Exception):
             _("Invalid Parameter: Encoding directive wasn't provided."))
 
 
-class DbMigrationError(DBError):
-
-    """Wrapped migration specific exception.
-
-    Raised when migrations couldn't be completed successfully.
-    """
-
-    def __init__(self, message=None):
-        super(DbMigrationError, self).__init__(message)
-
-
-class DBMigrationError(DbMigrationError):
+class DBMigrationError(DBError):
 
     """Wrapped migration specific exception.
 
@@ -297,29 +285,3 @@ class ContextNotRequestedError(AttributeError):
 
 class CantStartEngineError(Exception):
     """Error raised when the enginefacade cannot start up correctly."""
-
-
-class NotSupportedWarning(Warning):
-    """Warn that an argument or call that was passed is not supported.
-
-    This subclasses Warning so that it can be filtered as a distinct
-    category.
-
-    .. seealso::
-
-        https://docs.python.org/2/library/warnings.html
-
-    """
-
-
-class OsloDBDeprecationWarning(DeprecationWarning):
-    """Issued per usage of a deprecated API.
-
-    This subclasses DeprecationWarning so that it can be filtered as a distinct
-    category.
-
-    .. seealso::
-
-        https://docs.python.org/2/library/warnings.html
-
-    """
