@@ -176,7 +176,6 @@ class _TransactionFactory:
         }
         self._maker_cfg = {
             'expire_on_commit': _Default(False),
-            '__autocommit': _Default(False),
         }
         self._transaction_ctx_cfg = {
             'rollback_reader_sessions': False,
@@ -447,8 +446,6 @@ class _TransactionFactory:
 
     def _maker_args_for_conf(self, conf):
         maker_args = self._args_for_conf(self._maker_cfg, conf)
-        if '__autocommit' in maker_args:
-            maker_args['autocommit'] = maker_args.pop('__autocommit')
         return maker_args
 
     def dispose_pool(self):
@@ -1217,10 +1214,6 @@ class LegacyEngineFacade(object):
     :param sqlite_fk: enable foreign keys in SQLite
     :type sqlite_fk: bool
 
-    :param autocommit: use autocommit mode for created Session instances
-                       (only supported with SQLAlchemy < 2.0)
-    :type autocommit: bool
-
     :param expire_on_commit: expire session objects on commit
     :type expire_on_commit: bool
 
@@ -1258,8 +1251,7 @@ class LegacyEngineFacade(object):
 
     """
     def __init__(self, sql_connection, slave_connection=None,
-                 sqlite_fk=False, autocommit=False,
-                 expire_on_commit=False, _conf=None,
+                 sqlite_fk=False, expire_on_commit=False, _conf=None,
                  _factory=None, **kwargs):
         warnings.warn(
             "EngineFacade is deprecated; please use "
@@ -1267,14 +1259,6 @@ class LegacyEngineFacade(object):
             warning.OsloDBDeprecationWarning,
             stacklevel=2)
 
-        if autocommit is True:
-            warnings.warn(
-                'autocommit support has been removed in SQLAlchemy 2.0 and '
-                'should not be relied on; please rework your code to remove '
-                'reliance on this feature',
-                warning.OsloDBDeprecationWarning,
-                stacklevel=2)
-
         if _factory:
             self._factory = _factory
         else:
@@ -1282,7 +1266,6 @@ class LegacyEngineFacade(object):
 
             self._factory.configure(
                 sqlite_fk=sqlite_fk,
-                __autocommit=autocommit,
                 expire_on_commit=expire_on_commit,
                 **kwargs
             )
@@ -1348,7 +1331,7 @@ class LegacyEngineFacade(object):
 
     @classmethod
     def from_config(cls, conf,
-                    sqlite_fk=False, autocommit=False, expire_on_commit=False):
+                    sqlite_fk=False, expire_on_commit=False):
         """Initialize EngineFacade using oslo.config config instance options.
 
         :param conf: oslo.config config instance
@@ -1357,10 +1340,6 @@ class LegacyEngineFacade(object):
         :param sqlite_fk: enable foreign keys in SQLite
         :type sqlite_fk: bool
 
-        :param autocommit: use autocommit mode for created Session instances
-                           (only supported with SQLAlchemy < 2.0)
-        :type autocommit: bool
-
         :param expire_on_commit: expire session objects on commit
         :type expire_on_commit: bool
 
@@ -1369,5 +1348,4 @@ class LegacyEngineFacade(object):
         return cls(
             None,
             sqlite_fk=sqlite_fk,
-            autocommit=autocommit,
             expire_on_commit=expire_on_commit, _conf=conf)
