@@ -16,24 +16,25 @@
 """Core SQLAlchemy connectivity routines.
 """
 
+import functools
 import itertools
 import logging
 import os
 import re
 import time
 
+import debtcollector.removals
 import debtcollector.renames
-import six
 import sqlalchemy
 from sqlalchemy import event
 from sqlalchemy import exc
 from sqlalchemy import pool
-from sqlalchemy.sql.expression import select
+from sqlalchemy import select
 
 from oslo_db import exception
 
+from oslo_db.sqlalchemy import compat
 from oslo_db.sqlalchemy import exc_filters
-from oslo_db.sqlalchemy import ndb
 from oslo_db.sqlalchemy import utils
 
 LOG = logging.getLogger(__name__)
@@ -56,6 +57,13 @@ def _connect_ping_listener(connection, branch):
 
     Ping the server at transaction begin and transparently reconnect
     if a disconnect exception occurs.
+
+    This listener is used up until SQLAlchemy 2.0.5.  At 2.0.5, we use the
+    ``pool_pre_ping`` parameter instead of this event handler.
+
+    Note the current test suite in test_exc_filters still **tests** this
+    handler using all SQLAlchemy versions including 2.0.5 and greater.
+
     """
     if branch:
         return
@@ -69,8 +77,8 @@ def _connect_ping_listener(connection, branch):
     connection.should_close_with_result = False
     try:
         # run a SELECT 1.   use a core select() so that
-        # any details like that needed by Oracle, DB2 etc. are handled.
-        connection.scalar(select([1]))
+        # any details like that needed by the backend are handled.
+        connection.scalar(select(1))
     except exception.DBConnectionError:
         # catch DBConnectionError, which is raised by the filter
         # system.
@@ -80,9 +88,25 @@ def _connect_ping_listener(connection, branch):
         # run the select again to re-validate the Connection.
         LOG.exception(
             'Database connection was found disconnected; reconnecting')
-        connection.scalar(select([1]))
+        # TODO(ralonsoh): drop this attr check once SQLAlchemy minimum version
+        # is 2.0.
+        if hasattr(connection, 'rollback'):
+            connection.rollback()
+        connection.scalar(select(1))
     finally:
         connection.should_close_with_result = save_should_close_with_result
+        # TODO(ralonsoh): drop this attr check once SQLAlchemy minimum version
+        # is 2.0.
+        if hasattr(connection, 'rollback'):
+            connection.rollback()
+
+
+# SQLAlchemy 2.0 is compatible here, however oslo.db's test suite
+# raises for all deprecation errors, so we have to check for 2.0
+# and wrap out a parameter that is deprecated
+if compat.sqla_2:
+    _connect_ping_listener = functools.partial(
+        _connect_ping_listener, branch=False)
 
 
 def _setup_logging(connection_debug=0):
@@ -123,25 +147,33 @@ def _vet_url(url):
 
 
 @debtcollector.renames.renamed_kwarg(
-    "idle_timeout", "connection_recycle_time", replace=True)
+    'idle_timeout',
+    'connection_recycle_time',
+    replace=True,
+)
 def create_engine(sql_connection, sqlite_fk=False, mysql_sql_mode=None,
-                  mysql_enable_ndb=False,
+                  mysql_wsrep_sync_wait=None,
                   connection_recycle_time=3600,
                   connection_debug=0, max_pool_size=None, max_overflow=None,
                   pool_timeout=None, sqlite_synchronous=True,
                   connection_trace=False, max_retries=10, retry_interval=10,
                   thread_checkin=True, logging_name=None,
                   json_serializer=None,
-                  json_deserializer=None):
+                  json_deserializer=None, connection_parameters=None):
     """Return a new SQLAlchemy engine."""
 
-    url = sqlalchemy.engine.url.make_url(sql_connection)
+    url = utils.make_url(sql_connection)
+
+    if connection_parameters:
+        url = url.update_query_string(connection_parameters, append=True)
 
     _vet_url(url)
 
+    _native_pre_ping = compat.native_pre_ping_event_support
+
     engine_args = {
-        "pool_recycle": connection_recycle_time,
-        'convert_unicode': True,
+        'pool_recycle': connection_recycle_time,
+        'pool_pre_ping': _native_pre_ping,
         'connect_args': {},
         'logging_name': logging_name
     }
@@ -150,21 +182,21 @@ def create_engine(sql_connection, sqlite_fk=False, mysql_sql_mode=None,
 
     _init_connection_args(
         url, engine_args,
-        max_pool_size=max_pool_size,
-        max_overflow=max_overflow,
-        pool_timeout=pool_timeout,
-        json_serializer=json_serializer,
-        json_deserializer=json_deserializer,
+        dict(
+            max_pool_size=max_pool_size,
+            max_overflow=max_overflow,
+            pool_timeout=pool_timeout,
+            json_serializer=json_serializer,
+            json_deserializer=json_deserializer,
+        )
     )
 
     engine = sqlalchemy.create_engine(url, **engine_args)
 
-    if mysql_enable_ndb:
-        ndb.enable_ndb_support(engine)
-
     _init_events(
         engine,
         mysql_sql_mode=mysql_sql_mode,
+        mysql_wsrep_sync_wait=mysql_wsrep_sync_wait,
         sqlite_synchronous=sqlite_synchronous,
         sqlite_fk=sqlite_fk,
         thread_checkin=thread_checkin,
@@ -174,8 +206,10 @@ def create_engine(sql_connection, sqlite_fk=False, mysql_sql_mode=None,
     # register alternate exception handler
     exc_filters.register_engine(engine)
 
-    # register engine connect handler
-    event.listen(engine, "engine_connect", _connect_ping_listener)
+    if not _native_pre_ping:
+        # register engine connect handler.
+
+        event.listen(engine, "engine_connect", _connect_ping_listener)
 
     # initial connect + test
     # NOTE(viktors): the current implementation of _test_connection()
@@ -188,9 +222,16 @@ def create_engine(sql_connection, sqlite_fk=False, mysql_sql_mode=None,
 
 
 @utils.dispatch_for_dialect('*', multiple=True)
-def _init_connection_args(
-    url, engine_args,
-    max_pool_size=None, max_overflow=None, pool_timeout=None, **kw):
+def _init_connection_args(url, engine_args, kw):
+
+    # (zzzeek) kw is passed by reference rather than as **kw so that the
+    # init_connection_args routines can modify the contents of what
+    # will be passed to create_engine, including removing arguments that
+    # don't apply.  This allows things such as replacing QueuePool with
+    # NUllPool, for example, as the latter pool would reject these parameters.
+    max_pool_size = kw.get("max_pool_size", None)
+    max_overflow = kw.get("max_overflow", None)
+    pool_timeout = kw.get("pool_timeout", None)
 
     pool_class = url.get_dialect().get_pool_class(url)
     if issubclass(pool_class, pool.QueuePool):
@@ -203,17 +244,25 @@ def _init_connection_args(
 
 
 @_init_connection_args.dispatch_for("sqlite")
-def _init_connection_args(url, engine_args, **kw):
+def _init_connection_args(url, engine_args, kw):
     pool_class = url.get_dialect().get_pool_class(url)
-    # singletonthreadpool is used for :memory: connections;
-    # replace it with StaticPool.
     if issubclass(pool_class, pool.SingletonThreadPool):
+        # singletonthreadpool is used for :memory: connections;
+        # replace it with StaticPool.
         engine_args["poolclass"] = pool.StaticPool
         engine_args['connect_args']['check_same_thread'] = False
+    elif issubclass(pool_class, pool.QueuePool):
+        # SQLAlchemy 2.0 uses QueuePool for sqlite file DBs; put NullPool
+        # back to avoid compatibility issues
+        kw.pop("max_pool_size", None)
+        kw.pop("max_overflow", None)
+        engine_args.pop("max_pool_size", None)
+        engine_args.pop("max_overflow", None)
+        engine_args["poolclass"] = pool.NullPool
 
 
 @_init_connection_args.dispatch_for("postgresql")
-def _init_connection_args(url, engine_args, **kw):
+def _init_connection_args(url, engine_args, kw):
     if 'client_encoding' not in url.query:
         # Set encoding using engine_args instead of connect_args since
         # it's supported for PostgreSQL 8.*. More details at:
@@ -224,13 +273,13 @@ def _init_connection_args(url, engine_args, **kw):
 
 
 @_init_connection_args.dispatch_for("mysql")
-def _init_connection_args(url, engine_args, **kw):
+def _init_connection_args(url, engine_args, kw):
     if 'charset' not in url.query:
         engine_args['connect_args']['charset'] = 'utf8'
 
 
 @_init_connection_args.dispatch_for("mysql+mysqlconnector")
-def _init_connection_args(url, engine_args, **kw):
+def _init_connection_args(url, engine_args, kw):
     # mysqlconnector engine (<1.0) incorrectly defaults to
     # raise_on_warnings=True
     #  https://bitbucket.org/zzzeek/sqlalchemy/issue/2515
@@ -239,16 +288,12 @@ def _init_connection_args(url, engine_args, **kw):
 
 
 @_init_connection_args.dispatch_for("mysql+mysqldb")
-@_init_connection_args.dispatch_for("mysql+oursql")
-def _init_connection_args(url, engine_args, **kw):
+def _init_connection_args(url, engine_args, kw):
     # Those drivers require use_unicode=0 to avoid performance drop due
     # to internal usage of Python unicode objects in the driver
     #  http://docs.sqlalchemy.org/en/rel_0_9/dialects/mysql.html
     if 'use_unicode' not in url.query:
-        if six.PY3:
-            engine_args['connect_args']['use_unicode'] = 1
-        else:
-            engine_args['connect_args']['use_unicode'] = 0
+        engine_args['connect_args']['use_unicode'] = 1
 
 
 @utils.dispatch_for_dialect('*', multiple=True)
@@ -265,19 +310,26 @@ def _init_events(engine, thread_checkin=True, connection_trace=False, **kw):
 
 
 @_init_events.dispatch_for("mysql")
-def _init_events(engine, mysql_sql_mode=None, **kw):
+def _init_events(
+        engine, mysql_sql_mode=None, mysql_wsrep_sync_wait=None, **kw):
     """Set up event listeners for MySQL."""
 
-    if mysql_sql_mode is not None:
+    if mysql_sql_mode is not None or mysql_wsrep_sync_wait is not None:
         @sqlalchemy.event.listens_for(engine, "connect")
-        def _set_session_sql_mode(dbapi_con, connection_rec):
+        def _set_session_variables(dbapi_con, connection_rec):
             cursor = dbapi_con.cursor()
-            cursor.execute("SET SESSION sql_mode = %s", [mysql_sql_mode])
+            if mysql_sql_mode is not None:
+                cursor.execute("SET SESSION sql_mode = %s", [mysql_sql_mode])
+            if mysql_wsrep_sync_wait is not None:
+                cursor.execute(
+                    "SET SESSION wsrep_sync_wait = %s",
+                    [mysql_wsrep_sync_wait]
+                )
 
     @sqlalchemy.event.listens_for(engine, "first_connect")
     def _check_effective_sql_mode(dbapi_con, connection_rec):
-        if mysql_sql_mode is not None:
-            _set_session_sql_mode(dbapi_con, connection_rec)
+        if mysql_sql_mode is not None or mysql_wsrep_sync_wait is not None:
+            _set_session_variables(dbapi_con, connection_rec)
 
         cursor = dbapi_con.cursor()
         cursor.execute("SHOW VARIABLES LIKE 'sql_mode'")
@@ -295,9 +347,6 @@ def _init_events(engine, mysql_sql_mode=None, **kw):
                     "consider enabling TRADITIONAL or STRICT_ALL_TABLES",
                     realmode)
 
-    if ndb.ndb_status(engine):
-        ndb.init_ndb_events(engine)
-
 
 @_init_events.dispatch_for("sqlite")
 def _init_events(engine, sqlite_synchronous=True, sqlite_fk=False, **kw):
@@ -310,7 +359,7 @@ def _init_events(engine, sqlite_synchronous=True, sqlite_fk=False, **kw):
 
     def regexp(expr, item):
         reg = re.compile(expr)
-        return reg.search(six.text_type(item)) is not None
+        return reg.search(str(item)) is not None
 
     @sqlalchemy.event.listens_for(engine, "connect")
     def _sqlite_connect_events(dbapi_con, con_record):
@@ -338,7 +387,7 @@ def _init_events(engine, sqlite_synchronous=True, sqlite_fk=False, **kw):
         # emit our own BEGIN, checking for existing
         # transactional state
         if 'in_transaction' not in conn.info:
-            conn.execute("BEGIN")
+            conn.execute(sqlalchemy.text("BEGIN"))
             conn.info['in_transaction'] = True
 
     @sqlalchemy.event.listens_for(engine, "rollback")
@@ -352,7 +401,7 @@ def _test_connection(engine, max_retries, retry_interval):
     if max_retries == -1:
         attempts = itertools.count()
     else:
-        attempts = six.moves.range(max_retries)
+        attempts = range(max_retries)
     # See: http://legacy.python.org/dev/peps/pep-3110/#semantic-changes for
     # why we are not using 'de' directly (it can be removed from the local
     # scope).
@@ -367,7 +416,7 @@ def _test_connection(engine, max_retries, retry_interval):
             de_ref = de
     else:
         if de_ref is not None:
-            six.reraise(type(de_ref), de_ref)
+            raise de_ref
 
 
 def _add_process_guards(engine):
@@ -391,7 +440,6 @@ def _add_process_guards(engine):
                 "database connection, "
                 "which is being discarded and recreated.",
                 {"newproc": pid, "orig": connection_record.info['pid']})
-            connection_record.connection = connection_proxy.connection = None
             raise exc.DisconnectionError(
                 "Connection record belongs to pid %s, "
                 "attempting to check out in pid %s" %
