@@ -254,22 +254,6 @@ class DeletesFromSchema(ResetsData):
         """
 
 
-class RollsBackTransaction(ResetsData):
-    """Fixture class that maintains a database transaction per test.
-
-    """
-
-    def setup_for_reset(self, engine, facade):
-        conn = engine.connect()
-        engine = utils.NonCommittingEngine(conn)
-        self._reset_engine = enginefacade._TestTransactionFactory.apply_engine(
-            engine, facade)
-
-    def reset_schema_data(self, engine, facade):
-        self._reset_engine()
-        engine._dispose()
-
-
 class SimpleDbFixture(BaseDbFixture):
     """Fixture which provides an engine from a fixed URL.
 
@@ -376,7 +360,7 @@ class AdHocDbFixture(SimpleDbFixture):
     """
     def __init__(self, url=None):
         if url:
-            self.url = provision.sa_url.make_url(str(url))
+            self.url = utils.make_url(url)
             driver = self.url.get_backend_name()
         else:
             driver = None
@@ -562,9 +546,9 @@ def optimize_package_test_loader(file_):
 
     The function is invoked as::
 
-        from oslo_db.sqlalchemy import test_base
+        from oslo_db.sqlalchemy import test_fixtures
 
-        load_tests = test_base.optimize_package_test_loader(__file__)
+        load_tests = test_fixtures.optimize_package_test_loader(__file__)
 
     The loader *must* be present in the package level __init__.py.
 
@@ -602,9 +586,9 @@ def optimize_module_test_loader():
 
     The function is invoked as::
 
-        from oslo_db.sqlalchemy import test_base
+        from oslo_db.sqlalchemy import test_fixtures
 
-        load_tests = test_base.optimize_module_test_loader()
+        load_tests = test_fixtures.optimize_module_test_loader()
 
     The loader *must* be present in an individual module, and *not* the
     package level __init__.py.
