@@ -393,8 +393,8 @@ def update_returning_pk(query, values, surrogate_key):
     mapper = inspect(entity).mapper
     session = query.session
 
-    bind = session.connection(mapper=mapper)
-    if bind.dialect.implicit_returning:
+    bind = session.connection(bind_arguments=dict(mapper=mapper))
+    if bind.dialect.name == "postgresql":
         pk_strategy = _pk_strategy_returning
     elif bind.dialect.name == 'mysql' and \
         len(mapper.primary_key) == 1 and \
@@ -486,9 +486,11 @@ def _update_stmt_from_query(mapper, query, values):
     )
     primary_table = inspect(query.column_descriptions[0]['entity']).local_table
     where_criteria = query.whereclause
-    update_stmt = sql.update(primary_table,
-                             where_criteria,
-                             upd_values)
+    update_stmt = sql.update(
+        primary_table,
+    ).where(
+        where_criteria,
+    ).values(upd_values)
     return update_stmt
 
 
