@@ -18,19 +18,13 @@
 
 import collections
 from collections import abc
-import contextlib
 import itertools
 import logging
 import re
 
-from alembic.migration import MigrationContext
-from alembic.operations import Operations
-import debtcollector.removals
 from oslo_utils import timeutils
 import sqlalchemy
 from sqlalchemy import Boolean
-from sqlalchemy import CheckConstraint
-from sqlalchemy import Column
 from sqlalchemy.engine import Connectable
 from sqlalchemy.engine import url as sa_url
 from sqlalchemy import exc
@@ -42,14 +36,11 @@ from sqlalchemy import MetaData
 from sqlalchemy.sql.expression import cast
 from sqlalchemy.sql.expression import literal_column
 from sqlalchemy.sql import text
-from sqlalchemy import String
 from sqlalchemy import Table
-from sqlalchemy.types import NullType
 
 from oslo_db._i18n import _
 from oslo_db import exception
 from oslo_db.sqlalchemy import models
-from oslo_db.sqlalchemy import ndb
 
 # NOTE(ochuprykov): Add references for backwards compatibility
 InvalidSortKey = exception.InvalidSortKey
@@ -214,7 +205,7 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
             null_order_by_stmt = {
                 "": None,
                 "nullsfirst": sort_key_attr.is_(None),
-                "nullslast": sort_key_attr.isnot(None),
+                "nullslast": sort_key_attr.is_not(None),
             }[null_sort_dir]
         except KeyError:
             raise ValueError(_("Unknown sort direction, "
@@ -444,23 +435,6 @@ def get_table(engine, name):
     return Table(name, metadata, autoload_with=engine)
 
 
-def _get_not_supported_column(col_name_col_instance, column_name):
-    try:
-        column = col_name_col_instance[column_name]
-    except KeyError:
-        msg = _("Please specify column %s in col_name_col_instance "
-                "param. It is required because column has unsupported "
-                "type by SQLite.")
-        raise exception.ColumnError(msg % column_name)
-
-    if not isinstance(column, Column):
-        msg = _("col_name_col_instance param has wrong type of "
-                "column instance for column %s It should be instance "
-                "of sqlalchemy.Column.")
-        raise exception.ColumnError(msg % column_name)
-    return column
-
-
 def drop_old_duplicate_entries_from_table(engine, table_name,
                                           use_soft_delete, *uc_column_names):
     """Drop all old rows having the same values for columns in uc_columns.
@@ -520,273 +494,6 @@ def drop_old_duplicate_entries_from_table(engine, table_name,
             conn.execute(delete_statement)
 
 
-def _get_default_deleted_value(table):
-    if isinstance(table.c.id.type, Integer):
-        return 0
-    if isinstance(table.c.id.type, String):
-        return ""
-    raise exception.ColumnError(_("Unsupported id columns type"))
-
-
-def _restore_indexes_on_deleted_columns(engine, table_name, indexes):
-    table = get_table(engine, table_name)
-
-    real_indexes = get_indexes(engine, table_name)
-    existing_index_names = dict(
-        [(index['name'], index['column_names']) for index in real_indexes])
-
-    # NOTE(boris-42): Restore indexes on `deleted` column
-    for index in indexes:
-        if 'deleted' not in index['column_names']:
-            continue
-        name = index['name']
-        if name in existing_index_names:
-            column_names = [table.c[c] for c in existing_index_names[name]]
-            old_index = Index(name, *column_names, unique=index["unique"])
-            old_index.drop(engine)
-
-        column_names = [table.c[c] for c in index['column_names']]
-        new_index = Index(index["name"], *column_names, unique=index["unique"])
-        new_index.create(engine)
-
-
-@debtcollector.removals.remove(
-    message='This API is intended for use with sqlalchemy-migrate, support '
-    'for which is deprecated for removal; it will be removed in a future '
-    'release',
-    version='10.1.0',
-)
-def change_deleted_column_type_to_boolean(engine, table_name,
-                                          **col_name_col_instance):
-    if engine.name == "sqlite":
-        return _change_deleted_column_type_to_boolean_sqlite(
-            engine, table_name, **col_name_col_instance)
-    indexes = get_indexes(engine, table_name)
-    table = get_table(engine, table_name)
-
-    old_deleted = Column('old_deleted', Boolean, default=False)
-    table.metadata.bind = engine
-    try:
-        old_deleted.create(table, populate_default=False)
-    finally:
-        table.metadata.bind = None
-
-    with engine.connect() as conn, conn.begin():
-        conn.execute(
-            table.update().where(
-                table.c.deleted == table.c.id
-            ).values(old_deleted=True)
-        )
-
-    table.metadata.bind = engine
-    try:
-        table.c.deleted.drop()
-        table.c.old_deleted.alter(name="deleted")
-    finally:
-        table.metadata.bind = None
-
-    _restore_indexes_on_deleted_columns(engine, table_name, indexes)
-
-
-def _change_deleted_column_type_to_boolean_sqlite(engine, table_name,
-                                                  **col_name_col_instance):
-    table = get_table(engine, table_name)
-    columns = []
-    for column in table.columns:
-        column_copy = None
-        if column.name != "deleted":
-            if isinstance(column.type, NullType):
-                column_copy = _get_not_supported_column(col_name_col_instance,
-                                                        column.name)
-            else:
-                # FIXME(stephenfin): We shouldn't be using this private API;
-                # figure out how else to copy an arbitrary column schema
-                column_copy = column._copy()
-        else:
-            column_copy = Column('deleted', Boolean, default=0)
-        columns.append(column_copy)
-
-    # FIXME(stephenfin): We shouldn't be using this private API;
-    # figure out how else to copy an arbitrary column schema
-    constraints = [constraint._copy() for constraint in table.constraints]
-
-    with engine.connect() as conn:
-        meta = table.metadata
-        new_table = Table(
-            table_name + "__tmp__", meta,
-            *(columns + constraints))
-
-        with conn.begin():
-            new_table.create(conn)
-
-        indexes = []
-        for index in get_indexes(engine, table_name):
-            column_names = [new_table.c[c] for c in index['column_names']]
-            indexes.append(
-                Index(index["name"], *column_names, unique=index["unique"])
-            )
-
-        c_select = []
-        for c in table.c:
-            if c.name != "deleted":
-                c_select.append(c)
-            else:
-                c_select.append(table.c.deleted == table.c.id)
-
-        with conn.begin():
-            table.drop(conn)
-            for index in indexes:
-                index.create(conn)
-
-        table.metadata.bind = engine
-        try:
-            new_table.rename(table_name)
-        finally:
-            table.metadata.bind = None
-
-        with conn.begin():
-            conn.execute(
-                new_table.update().where(
-                    new_table.c.deleted == new_table.c.id
-                ).values(deleted=True)
-            )
-
-
-@debtcollector.removals.remove(
-    message='This API is intended for use with sqlalchemy-migrate, support '
-    'for which is deprecated for removal; it will be removed in a future '
-    'release',
-    version='10.1.0',
-)
-def change_deleted_column_type_to_id_type(engine, table_name,
-                                          **col_name_col_instance):
-    if engine.name == "sqlite":
-        return _change_deleted_column_type_to_id_type_sqlite(
-            engine, table_name, **col_name_col_instance)
-    indexes = get_indexes(engine, table_name)
-    table = get_table(engine, table_name)
-
-    new_deleted = Column('new_deleted', table.c.id.type,
-                         default=_get_default_deleted_value(table))
-    table.metadata.bind = engine
-    try:
-        new_deleted.create(table, populate_default=True)
-    finally:
-        table.metadata.bind = None
-
-    table.metadata.bind = engine
-    try:
-        with engine.connect() as conn, conn.begin():
-            deleted = True  # workaround for pyflakes
-            conn.execute(
-                table.update().where(
-                    table.c.deleted == deleted
-                ).values(new_deleted=table.c.id)
-            )
-
-        table.c.deleted.drop()
-        table.c.new_deleted.alter(name="deleted")
-
-        _restore_indexes_on_deleted_columns(engine, table_name, indexes)
-    finally:
-        table.metadata.bind = None
-
-
-def _is_deleted_column_constraint(constraint):
-    # NOTE(boris-42): There is no other way to check is CheckConstraint
-    #                 associated with deleted column.
-    if not isinstance(constraint, CheckConstraint):
-        return False
-    sqltext = str(constraint.sqltext)
-    # NOTE(zzzeek): SQLite never reflected CHECK contraints here
-    # in any case until version 1.1.   Safe to assume that any CHECK
-    # that's talking about the value of "deleted in (something)" is
-    # the boolean constraint we're looking to get rid of.
-    return bool(re.match(r".*deleted in \(.*\)", sqltext, re.I))
-
-
-def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
-                                                  **col_name_col_instance):
-    # NOTE(boris-42): sqlalchemy-migrate can't drop column with check
-    # constraints in sqlite DB and our `deleted` column has two check
-    # constraints. There is only one way to remove these constraints:
-    #
-    # 1) Create new table with the same columns, constraints and indexes.
-    #    (except deleted column).
-    # 2) Copy all data from old to new table.
-    # 3) Drop old table.
-    # 4) Rename new table to old table name.
-    meta = MetaData()
-    table = Table(table_name, meta, autoload_with=engine)
-    default_deleted_value = _get_default_deleted_value(table)
-
-    columns = []
-    for column in table.columns:
-        column_copy = None
-        if column.name != "deleted":
-            if isinstance(column.type, NullType):
-                column_copy = _get_not_supported_column(col_name_col_instance,
-                                                        column.name)
-            else:
-                # FIXME(stephenfin): We shouldn't be using this private API;
-                # figure out how else to copy an arbitrary column schema
-                column_copy = column._copy()
-        else:
-            column_copy = Column('deleted', table.c.id.type,
-                                 default=default_deleted_value)
-        columns.append(column_copy)
-
-    constraints = []
-    for constraint in table.constraints:
-        if not _is_deleted_column_constraint(constraint):
-            # FIXME(stephenfin): We shouldn't be using this private API;
-            # figure out how else to copy an arbitrary constraint schema
-            constraints.append(constraint._copy())
-
-    with engine.connect() as conn:
-        # we need separate transactions, since we must create the table before
-        # we can copy entries into it (later)
-        with conn.begin():
-            new_table = Table(
-                table_name + "__tmp__", meta,
-                *(columns + constraints))
-            new_table.create(conn)
-
-        indexes = []
-        for index in get_indexes(engine, table_name):
-            column_names = [new_table.c[c] for c in index['column_names']]
-            indexes.append(
-                Index(index["name"], *column_names, unique=index["unique"])
-            )
-
-        with conn.begin():
-            table.drop(conn)
-            for index in indexes:
-                index.create(conn)
-
-        with conn.begin():
-            new_table.metadata.bind = engine
-            try:
-                new_table.rename(table_name)
-            finally:
-                new_table.metadata.bind = None
-
-            deleted = True  # workaround for pyflakes
-            conn.execute(
-                new_table.update().where(
-                    new_table.c.deleted == deleted
-                ).values(deleted=new_table.c.id)
-            )
-
-            # NOTE(boris-42): Fix value of deleted column: False -> "" or 0.
-            deleted = False  # workaround for pyflakes
-            conn.execute(
-                new_table.update().where(
-                    new_table.c.deleted == deleted
-                ).values(deleted=default_deleted_value)
-            )
-
-
 def get_db_connection_info(conn_pieces):
     database = conn_pieces.path.strip('/')
     loc_pieces = conn_pieces.netloc.split('@')
@@ -1136,36 +843,6 @@ def get_non_innodb_tables(connectable, skip_tables=('migrate_version',
     return [i[0] for i in noninnodb]
 
 
-def get_non_ndbcluster_tables(connectable, skip_tables=None):
-    """Get a list of tables which don't use MySQL Cluster (NDB) storage engine.
-
-    :param connectable: a SQLAlchemy Engine or Connection instance
-    :param skip_tables: a list of tables which might have a different
-                        storage engine
-    """
-    query_str = """
-        SELECT table_name
-        FROM information_schema.tables
-        WHERE table_schema = :database AND
-              engine != 'ndbcluster'
-    """
-
-    params = {}
-    if skip_tables:
-        params = dict(
-            ('skip_%s' % i, table_name)
-            for i, table_name in enumerate(skip_tables)
-        )
-
-        placeholders = ', '.join(':' + p for p in params)
-        query_str += ' AND table_name NOT IN (%s)' % placeholders
-
-    params['database'] = connectable.engine.url.database
-    query = text(query_str)
-    nonndbcluster = connectable.execute(query, **params)
-    return [i[0] for i in nonndbcluster]
-
-
 def get_foreign_key_constraint_name(engine, table_name, column_name):
     """Find the name of foreign key in a table, given constrained column name.
 
@@ -1185,79 +862,9 @@ def get_foreign_key_constraint_name(engine, table_name, column_name):
             return fk['name']
 
 
-@contextlib.contextmanager
-def suspend_fk_constraints_for_col_alter(
-        engine, table_name, column_name, referents=[]):
-    """Detect foreign key constraints, drop, and recreate.
-
-    This is used to guard against a column ALTER that on some backends
-    cannot proceed unless foreign key constraints are not present.
-
-    e.g.::
-
-        from oslo_db.sqlalchemy.util import (
-            suspend_fk_constraints_for_col_alter
-        )
-
-        with suspend_fk_constraints_for_col_alter(
-            migrate_engine, "user_table",
-            referents=[
-                "local_user", "nonlocal_user", "project"
-            ]):
-            user_table.c.domain_id.alter(nullable=False)
-
-    :param engine: a SQLAlchemy engine (or connection)
-
-    :param table_name: target table name.  All foreign key constraints
-     that refer to the table_name / column_name will be dropped and recreated.
-
-    :param column_name: target column name.  all foreign key constraints
-     which refer to this column, either partially or fully, will be dropped
-     and recreated.
-
-    :param referents: sequence of string table names to search for foreign
-     key constraints.   A future version of this function may no longer
-     require this argument, however for the moment it is required.
-
-    """
-    if (
-        not ndb.ndb_status(engine)
-    ):
-        yield
+def make_url(target):
+    """Return a ``url.URL`` object"""
+    if isinstance(target, (str, sa_url.URL)):
+        return sa_url.make_url(target)
     else:
-        with engine.connect() as conn:
-            insp = inspect(conn)
-            fks = []
-            for ref_table_name in referents:
-                for fk in insp.get_foreign_keys(ref_table_name):
-                    if not fk.get('name'):
-                        raise AssertionError("foreign key hasn't a name.")
-                    if fk['referred_table'] == table_name and \
-                            column_name in fk['referred_columns']:
-                        fk['source_table'] = ref_table_name
-                        if 'options' not in fk:
-                            fk['options'] = {}
-                        fks.append(fk)
-
-            ctx = MigrationContext.configure(conn)
-            op = Operations(ctx)
-
-            with conn.begin():
-                for fk in fks:
-                    op.drop_constraint(
-                        fk['name'], fk['source_table'], type_="foreignkey")
-
-            yield
-
-            with conn.begin():
-                for fk in fks:
-                    op.create_foreign_key(
-                        fk['name'], fk['source_table'],
-                        fk['referred_table'],
-                        fk['constrained_columns'],
-                        fk['referred_columns'],
-                        onupdate=fk['options'].get('onupdate'),
-                        ondelete=fk['options'].get('ondelete'),
-                        deferrable=fk['options'].get('deferrable'),
-                        initially=fk['options'].get('initially'),
-                    )
+        return sa_url.make_url(str(target))
