@@ -16,28 +16,34 @@
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy.test_base import backend_specific  # noqa
 from oslo_db.sqlalchemy import test_fixtures as db_fixtures
-from oslotest import base as test_base
+from oslo_db.tests import base as test_base
 
 
 @enginefacade.transaction_context_provider
 class Context(object):
     pass
 
+
 context = Context()
 
 
-class DbTestCase(db_fixtures.OpportunisticDBTestMixin, test_base.BaseTestCase):
+# NOTE (zzzeek) These test classes are **private to oslo.db**.  Please
+# make use of oslo_db.sqlalchemy.test_fixtures directly.
+
+class _DbTestCase(
+    db_fixtures.OpportunisticDBTestMixin, test_base.BaseTestCase,
+):
 
     def setUp(self):
-        super(DbTestCase, self).setUp()
+        super(_DbTestCase, self).setUp()
 
         self.engine = enginefacade.writer.get_engine()
         self.sessionmaker = enginefacade.writer.get_sessionmaker()
 
 
-class MySQLOpportunisticTestCase(DbTestCase):
+class _MySQLOpportunisticTestCase(_DbTestCase):
     FIXTURE = db_fixtures.MySQLOpportunisticFixture
 
 
-class PostgreSQLOpportunisticTestCase(DbTestCase):
+class _PostgreSQLOpportunisticTestCase(_DbTestCase):
     FIXTURE = db_fixtures.PostgresqlOpportunisticFixture
