@@ -15,13 +15,14 @@
 
 """Unit tests for DB API."""
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_utils import importutils
 
 from oslo_db import api
 from oslo_db import exception
-from oslo_db.tests import utils as test_utils
+from oslo_db.tests import base as test_base
 
 sqla = importutils.try_import('sqlalchemy')
 if not sqla:
@@ -64,7 +65,8 @@ class DBAPI(object):
         return args, kwargs
 
 
-class DBAPITestCase(test_utils.BaseTestCase):
+class DBAPITestCase(test_base.BaseTestCase):
+
     def test_dbapi_full_path_module_method(self):
         dbapi = api.DBAPI('oslo_db.tests.test_api')
         result = dbapi.api_class_call1(1, 2, kwarg1='meow')
@@ -90,8 +92,9 @@ class DBAPITestCase(test_utils.BaseTestCase):
 
 
 class DBReconnectTestCase(DBAPITestCase):
+
     def setUp(self):
-        super(DBReconnectTestCase, self).setUp()
+        super().setUp()
 
         self.test_db_api = DBAPI()
         patcher = mock.patch(__name__ + '.get_backend',
@@ -199,6 +202,7 @@ class DBReconnectTestCase(DBAPITestCase):
 
 
 class DBRetryRequestCase(DBAPITestCase):
+
     def test_retry_wrapper_succeeds(self):
         @api.wrap_db_retry(max_retries=10)
         def some_method():
@@ -206,7 +210,8 @@ class DBRetryRequestCase(DBAPITestCase):
 
         some_method()
 
-    def test_retry_wrapper_reaches_limit(self):
+    @mock.patch('oslo_db.api.time.sleep', return_value=None)
+    def test_retry_wrapper_reaches_limit(self, mock_sleep):
         max_retries = 2
 
         @api.wrap_db_retry(max_retries=max_retries)
@@ -218,7 +223,8 @@ class DBRetryRequestCase(DBAPITestCase):
         self.assertRaises(ValueError, some_method, res)
         self.assertEqual(max_retries + 1, res['result'])
 
-    def test_retry_wrapper_exception_checker(self):
+    @mock.patch('oslo_db.api.time.sleep', return_value=None)
+    def test_retry_wrapper_exception_checker(self, mock_sleep):
 
         def exception_checker(exc):
             return isinstance(exc, ValueError) and exc.args[0] < 5
@@ -253,3 +259,61 @@ class DBRetryRequestCase(DBAPITestCase):
 
         self.assertRaises(AttributeError, some_method)
         self.assertFalse(mock_log.called)
+
+    @mock.patch('oslo_db.api.time.sleep', return_value=None)
+    def test_retry_wrapper_deadlock(self, mock_sleep):
+
+        # Tests that jitter is False, if the retry wrapper hits a
+        # non-deadlock error
+        @api.wrap_db_retry(max_retries=1, retry_on_deadlock=True)
+        def some_method_no_deadlock():
+            raise exception.RetryRequest(ValueError())
+        with mock.patch(
+                'oslo_db.api.wrap_db_retry._get_inc_interval') as mock_get:
+            mock_get.return_value = 2, 2
+            self.assertRaises(ValueError, some_method_no_deadlock)
+            mock_get.assert_called_once_with(1, False)
+
+        # Tests that jitter is True, if the retry wrapper hits a deadlock
+        # error.
+        @api.wrap_db_retry(max_retries=1, retry_on_deadlock=True)
+        def some_method_deadlock():
+            raise exception.DBDeadlock('test')
+        with mock.patch(
+                'oslo_db.api.wrap_db_retry._get_inc_interval') as mock_get:
+            mock_get.return_value = 0.1, 2
+            self.assertRaises(exception.DBDeadlock, some_method_deadlock)
+            mock_get.assert_called_once_with(1, True)
+
+        # Tests that jitter is True, if the jitter is enable by user
+        @api.wrap_db_retry(max_retries=1, retry_on_deadlock=True, jitter=True)
+        def some_method_no_deadlock_exp():
+            raise exception.RetryRequest(ValueError())
+        with mock.patch(
+                'oslo_db.api.wrap_db_retry._get_inc_interval') as mock_get:
+            mock_get.return_value = 0.1, 2
+            self.assertRaises(ValueError, some_method_no_deadlock_exp)
+            mock_get.assert_called_once_with(1, True)
+
+    def test_wrap_db_retry_get_interval(self):
+        x = api.wrap_db_retry(max_retries=5, retry_on_deadlock=True,
+                              max_retry_interval=11)
+        self.assertEqual(11, x.max_retry_interval)
+
+        for i in (1, 2, 4):
+            # With jitter: sleep_time = [0, 2 ** retry_times)
+            sleep_time, n = x._get_inc_interval(i, True)
+            self.assertEqual(2 * i, n)
+            self.assertTrue(2 * i > sleep_time)
+            # Without jitter: sleep_time = 2 ** retry_times
+            sleep_time, n = x._get_inc_interval(i, False)
+            self.assertEqual(2 * i, n)
+            self.assertEqual(2 * i, sleep_time)
+
+        for i in (8, 16, 32):
+            sleep_time, n = x._get_inc_interval(i, False)
+            self.assertEqual(x.max_retry_interval, sleep_time)
+            self.assertEqual(2 * i, n)
+            sleep_time, n = x._get_inc_interval(i, True)
+            self.assertTrue(x.max_retry_interval >= sleep_time)
+            self.assertEqual(2 * i, n)
